@@ -9,8 +9,10 @@
 #define _BITFIELDMETER_H_
 
 #include "meter.h"
+#include "xosview.h"
 #include "timer.h"
 
+
 class BitFieldMeter : public Meter {
 public:
   BitFieldMeter( XOSView *parent, int numBits = 1, int numfields = 1,
@@ -19,8 +21,8 @@ public:
 	      int docaptions = 0, int dolegends = 0, int dousedlegends = 0 );
   virtual ~BitFieldMeter( void );
 
-  virtual void drawfields( int manditory = 0 );
-  void drawBits( int manditory = 0 );
+  virtual void drawfields( int mandatory = 0 );
+  void drawBits( int mandatory = 0 );
 
   void setfieldcolor( int field, const char *color );
   void setfieldcolor( int field, unsigned long color);
@@ -50,6 +52,8 @@ protected:
   UsedType print_;
   int printedZeroTotalMesg_;
   int numWarnings_;
+  bool metric_;
+  int usedoffset_;
 
   unsigned long onColor_, offColor_;
   char *bits_, *lastbits_;
@@ -57,7 +61,7 @@ protected:
 
   void SetUsedFormat ( const char * const str );
   void drawfieldlegend( void );
-  void drawused( int manditory );
+  void drawused( int mandatory );
   bool checkX(int x, int width) const;
 
   void setNumFields(int n);
