@@ -17,17 +17,22 @@
 #define _CPUMETER_H_
 
 #include "fieldmetergraph.h"
+#include "xosview.h"
+#include "defines.h"
 
-#if defined(XOSVIEW_DFBSD)
-#define CPUSTATES 5
-#else
+// for CPUSTATES
+#if defined(XOSVIEW_NETBSD)
+#include <sys/sched.h>
+#elif defined(XOSVIEW_OPENBSD)
 #include <sys/dkstat.h>
+#else
+#include <sys/resource.h>
 #endif
 
 
 class CPUMeter : public FieldMeterGraph {
 public:
-	CPUMeter( XOSView *parent );
+	CPUMeter( XOSView *parent, unsigned int nbr );
 	~CPUMeter( void );
 
 	const char *name( void ) const { return "CPUMeter"; }
@@ -38,8 +43,8 @@ protected:
 	void getcputime( void );
 
 private:
-	long cputime_[2][CPUSTATES];
-	int cpuindex_;
+	uint64_t cputime_[2][CPUSTATES];
+	unsigned int cpuindex_, nbr_;
 };
 
 
