@@ -8,11 +8,11 @@
 //    authors for a copy.
 //
 
+#include "intratemeter.h"
+#include "kernel.h"
 #include <stdlib.h>
 #include <strings.h>
 #include <err.h>
-#include "kernel.h"
-#include "intratemeter.h"
 
 
 IrqRateMeter::IrqRateMeter( XOSView *parent )
@@ -23,8 +23,8 @@ IrqRateMeter::IrqRateMeter( XOSView *parent )
 		disableMeter();
 	}
 	irqcount_ = BSDNumInts();
-	irqs_ = (unsigned long *)calloc(irqcount_ + 1, sizeof(long));
-	lastirqs_ = (unsigned long *)calloc(irqcount_ + 1, sizeof(long));
+	irqs_ = (uint64_t *)calloc(irqcount_ + 1, sizeof(uint64_t));
+	lastirqs_ = (uint64_t *)calloc(irqcount_ + 1, sizeof(uint64_t));
 }
 
 IrqRateMeter::~IrqRateMeter( void ) {
