@@ -43,8 +43,10 @@
 #if defined(XOSVIEW_FREEBSD) || defined(XOSVIEW_DFBSD)
 static const char ACPIDEV[] = "/dev/acpi";
 static const char APMDEV[] = "/dev/apm";
+static int maxcpus = 1;
 #include <net/if_var.h>
 #include <sys/ioctl.h>
+#include <sys/resource.h>
 #include <dev/acpica/acpiio.h>
 #include <machine/apm_bios.h>
 #endif
@@ -62,11 +64,13 @@ static int mib_dsk[3] = { CTL_HW, HW_IOSTATS, sizeof(struct io_sysctl) };
 #if defined(XOSVIEW_OPENBSD)
 #include <sys/sched.h>
 #include <sys/disk.h>
+#include <sys/dkstat.h>
 #include <sys/mount.h>
 #include <net/route.h>
 #include <net/if_dl.h>
 static int mib_spd[2] = { CTL_HW, HW_CPUSPEED };
 static int mib_cpt[2] = { CTL_KERN, KERN_CPTIME };
+static int mib_cpt2[3] = { CTL_KERN, KERN_CPTIME2, 0 };
 static int mib_ifl[6] = { CTL_NET, AF_ROUTE, 0, 0, NET_RT_IFLIST, 0 };
 #endif
 
@@ -76,11 +80,6 @@ static int mib_sen[5] = { CTL_HW, HW_SENSORS };
 #endif
 
 #if defined(HAVE_DEVSTAT)
-#if defined(XOSVIEW_DFBSD)
-#define CPUSTATES 5
-#else
-#include <sys/dkstat.h>
-#endif
 #include <devstat.h>
 #endif
 
@@ -178,7 +177,7 @@ static char kernelFileName[_POSIX2_LINE_MAX];
 //  Since this is C++, it's an inline function rather than a macro.
 
 static inline void
-safe_kvm_read(u_long kernel_addr, void* user_addr, size_t nbytes) {
+safe_kvm_read(unsigned long kernel_addr, void* user_addr, size_t nbytes) {
 	/*  Check for obvious bad symbols (i.e., from /netbsd when we
 	 *  booted off of /netbsd.old), such as symbols that reference
 	 *  0x00000000 (or anywhere in the first 256 bytes of memory).  */
@@ -248,14 +247,16 @@ OpenKDIfNeeded() {
 	//  Look at all of the returned symbols, and check for bad lookups.
 	//  (This may be unnecessary, but better to check than not to...  )
 	struct nlist *nlp = nlst;
-	while (nlp && nlp->n_name && strncmp(nlp->n_name, DUMMY_SYM, strlen(DUMMY_SYM))) {
-		if ( nlp->n_type == 0 || nlp->n_value == 0 )
+	while (nlp && nlp->n_name) {
+		if ( strncmp(nlp->n_name, DUMMY_SYM, strlen(DUMMY_SYM))) {
+			if ( nlp->n_type == 0 || nlp->n_value == 0 )
 #if defined(XOSVIEW_FREEBSD) && defined(__alpha__)
-			/* XXX: this should be properly fixed. */
-			;
+				/* XXX: this should be properly fixed. */
+				;
 #else
-			warnx("kvm_nlist() lookup failed for symbol '%s'.", nlp->n_name);
+				warnx("kvm_nlist() lookup failed for symbol '%s'.", nlp->n_name);
 #endif
+		}
 		nlp++;
 	}
 }
@@ -308,7 +309,7 @@ BSDPageInit() {
 /* meminfo[5]  = { active, inactive, wired, cached, free } */
 /* pageinfo[2] = { pages_in, pages_out }                   */
 void
-BSDGetPageStats(u_int64_t *meminfo, u_int64_t *pageinfo) {
+BSDGetPageStats(uint64_t *meminfo, uint64_t *pageinfo) {
 #if defined(HAVE_UVM)
 #ifdef VM_UVMEXP2
 	struct uvmexp_sysctl uvm;
@@ -321,18 +322,18 @@ BSDGetPageStats(u_int64_t *meminfo, u_int64_t *pageinfo) {
 
 	if (meminfo) {
 		// UVM excludes kernel memory -> assume it is active mem
-		meminfo[0] = (u_int64_t)(uvm.npages - uvm.inactive - uvm.wired - uvm.free) * uvm.pagesize;
-		meminfo[1] = (u_int64_t)uvm.inactive * uvm.pagesize;
-		meminfo[2] = (u_int64_t)uvm.wired * uvm.pagesize;
+		meminfo[0] = (uint64_t)(uvm.npages - uvm.inactive - uvm.wired - uvm.free) * uvm.pagesize;
+		meminfo[1] = (uint64_t)uvm.inactive * uvm.pagesize;
+		meminfo[2] = (uint64_t)uvm.wired * uvm.pagesize;
 
 		// cache is already included in active and inactive memory and
 		// there's no way to know how much is in which -> disable cache
 		meminfo[3] = 0;
-		meminfo[4] = (u_int64_t)uvm.free * uvm.pagesize;
+		meminfo[4] = (uint64_t)uvm.free * uvm.pagesize;
 	}
 	if (pageinfo) {
-		pageinfo[0] = (u_int64_t)uvm.pgswapin;
-		pageinfo[1] = (u_int64_t)uvm.pgswapout;
+		pageinfo[0] = (uint64_t)uvm.pgswapin;
+		pageinfo[1] = (uint64_t)uvm.pgswapout;
 	}
 #else  /* HAVE_UVM */
 	struct vmmeter vm;
@@ -349,22 +350,22 @@ BSDGetPageStats(u_int64_t *meminfo, u_int64_t *pageinfo) {
 #endif
 	if (meminfo) {
 #if defined(XOSVIEW_FREEBSD)
-		meminfo[0] = (u_int64_t)vm.v_active_count * vm.v_page_size;
-		meminfo[1] = (u_int64_t)vm.v_inactive_count * vm.v_page_size;
-		meminfo[2] = (u_int64_t)vm.v_wire_count * vm.v_page_size;
-		meminfo[3] = (u_int64_t)vm.v_cache_count * vm.v_page_size;
-		meminfo[4] = (u_int64_t)vm.v_free_count * vm.v_page_size;
+		meminfo[0] = (uint64_t)vm.v_active_count * vm.v_page_size;
+		meminfo[1] = (uint64_t)vm.v_inactive_count * vm.v_page_size;
+		meminfo[2] = (uint64_t)vm.v_wire_count * vm.v_page_size;
+		meminfo[3] = (uint64_t)vm.v_cache_count * vm.v_page_size;
+		meminfo[4] = (uint64_t)vm.v_free_count * vm.v_page_size;
 #else  /* XOSVIEW_DFBSD */
-		meminfo[0] = (u_int64_t)vms.v_active_count * vms.v_page_size;
-		meminfo[1] = (u_int64_t)vms.v_inactive_count * vms.v_page_size;
-		meminfo[2] = (u_int64_t)vms.v_wire_count * vms.v_page_size;
-		meminfo[3] = (u_int64_t)vms.v_cache_count * vms.v_page_size;
-		meminfo[4] = (u_int64_t)vms.v_free_count * vms.v_page_size;
+		meminfo[0] = (uint64_t)vms.v_active_count * vms.v_page_size;
+		meminfo[1] = (uint64_t)vms.v_inactive_count * vms.v_page_size;
+		meminfo[2] = (uint64_t)vms.v_wire_count * vms.v_page_size;
+		meminfo[3] = (uint64_t)vms.v_cache_count * vms.v_page_size;
+		meminfo[4] = (uint64_t)vms.v_free_count * vms.v_page_size;
 #endif
 	}
 	if (pageinfo) {
-		pageinfo[0] = (u_int64_t)vm.v_vnodepgsin + (u_int64_t)vm.v_swappgsin;
-		pageinfo[1] = (u_int64_t)vm.v_vnodepgsout + (u_int64_t)vm.v_swappgsout;
+		pageinfo[0] = (uint64_t)vm.v_vnodepgsin + (uint64_t)vm.v_swappgsin;
+		pageinfo[1] = (uint64_t)vm.v_vnodepgsout + (uint64_t)vm.v_swappgsout;
 	}
 #endif
 }
@@ -375,45 +376,91 @@ BSDGetPageStats(u_int64_t *meminfo, u_int64_t *pageinfo) {
 void
 BSDCPUInit() {
 	OpenKDIfNeeded();
+#if defined(XOSVIEW_FREEBSD)
+	size_t size = sizeof(maxcpus);
+	if ( sysctlbyname("kern.smp.maxcpus", &maxcpus, &size, NULL, 0) < 0 )
+		err(EX_OSERR, "sysctl kern.smp.maxcpus failed");
+#elif defined(XOSVIEW_DFBSD)
+	if ( kinfo_get_cpus(&maxcpus) )
+		err(EX_OSERR, "kinfo_get_cpus() failed");
+#endif
 }
 
 void
-#if defined(XOSVIEW_NETBSD) || defined(XOSVIEW_DFBSD)
-BSDGetCPUTimes(u_int64_t *timeArray) {
-#else
-BSDGetCPUTimes(long *timeArray) {
-#endif
-#if defined(XOSVIEW_DFBSD)
-	struct kinfo_cputime cpu;
-#elif defined(XOSVIEW_NETBSD)
-	uint64_t cpu[CPUSTATES];
-#else
-	long cpu[CPUSTATES];
-#endif
+BSDGetCPUTimes(uint64_t *timeArray, unsigned int cpu) {
+	// timeArray is CPUSTATES long.
+	// cpu is the number of CPU to return, starting from 1. If cpu == 0,
+	// return aggregate times for all CPUs.
+	// All BSDs have separate calls for aggregate and separate times. Only
+	// OpenBSD returns one CPU per call, others return all at once.
 	if (!timeArray)
-		errx(EX_SOFTWARE, "BSDGetCPUTimes(): passed pointer was null.");
-	if (CPUSTATES != 5)
-		errx(EX_SOFTWARE, "xosview for *BSD expects 5 cpu states.");
+		err(EX_SOFTWARE, "BSDGetCPUTimes(): passed pointer was null.");
+	size_t size;
+#if defined(XOSVIEW_DFBSD)
+	size = sizeof(struct kinfo_cputime);
+	struct kinfo_cputime *times = (struct kinfo_cputime *)calloc(maxcpus + 1, size);
+#elif defined(XOSVIEW_NETBSD)
+	size = CPUSTATES * sizeof(uint64_t);
+	uint64_t *times = (uint64_t*)calloc(BSDCountCpus() + 1, size);
+#elif defined(XOSVIEW_FREEBSD)
+	size = CPUSTATES * sizeof(long);
+	long *times = (long*)calloc(maxcpus + 1, size);
+#else // XOSVIEW_OPENBSD
+	uint64_t *times = (uint64_t*)calloc(CPUSTATES, sizeof(uint64_t));
+#endif
+	// this array will have aggregate values at 0, then each CPU (except on
+	// OpenBSD), so that cpu can be used as index
+	if (!times)
+		err(EX_OSERR, "BSDGetCPUTimes(): malloc failed");
 
 #if defined(XOSVIEW_DFBSD)
-	if (kinfo_get_sched_cputime(&cpu))
-		err(EX_OSERR, "kinfo_get_sched_cputime() failed");
-	timeArray[0] = cpu.cp_user;
-	timeArray[1] = cpu.cp_nice;
-	timeArray[2] = cpu.cp_sys;
-	timeArray[3] = cpu.cp_intr;
-	timeArray[4] = cpu.cp_idle;
-#else
-	size_t size = sizeof(cpu);
-#if defined(XOSVIEW_NETBSD) || defined(XOSVIEW_OPENBSD)
-	if ( sysctl(mib_cpt, 2, cpu, &size, NULL, 0) < 0 )
-#else
-	if ( sysctlbyname("kern.cp_time", cpu, &size, NULL, 0) < 0 )
+	if (cpu == 0) {
+		if (kinfo_get_sched_cputime(times))
+			err(EX_OSERR, "kinfo_get_sched_cputime() failed");
+	}
+	else {
+		size = maxcpus * sizeof(times[0]);
+		if ( sysctlbyname("kern.cputime", times + 1, &size, NULL, 0) < 0 )
+			err(EX_OSERR, "sysctl kern.cputime failed");
+	}
+	timeArray[0] = times[cpu].cp_user;
+	timeArray[1] = times[cpu].cp_nice;
+	timeArray[2] = times[cpu].cp_sys;
+	timeArray[3] = times[cpu].cp_intr;
+	timeArray[4] = times[cpu].cp_idle;
+#else  // !XOSVIEW_DFBSD
+	size = CPUSTATES * sizeof(times[0]);
+	if (cpu == 0) {  // aggregate times
+#if defined(XOSVIEW_FREEBSD)
+		if ( sysctlbyname("kern.cp_time", times, &size, NULL, 0) < 0 )
+#else  // XOSVIEW_NETBSD || XOSVIEW_OPENBSD
+		if ( sysctl(mib_cpt, 2, times, &size, NULL, 0) < 0 )
 #endif
-		err(EX_OSERR, "sysctl kern.cp_time failed");
+			err(EX_OSERR, "sysctl kern.cp_time failed");
+	}
+	else {  // separate times
+#if defined(XOSVIEW_FREEBSD)
+		size *= maxcpus;
+		if ( sysctlbyname("kern.cp_times", times + CPUSTATES, &size, NULL, 0) < 0 )
+			err(EX_OSERR, "sysctl kern.cp_times failed");
+#elif defined(XOSVIEW_NETBSD)
+		size *= BSDCountCpus();
+		if ( sysctl(mib_cpt, 2, times + CPUSTATES, &size, NULL, 0) < 0 )
+			err(EX_OSERR, "sysctl kern.cp_time failed");
+#else  // XOSVIEW_OPENBSD
+		mib_cpt2[2] = cpu - 1;
+		if ( sysctl(mib_cpt2, 3, times, &size, NULL, 0) < 0 )
+			err(EX_OSERR, "sysctl kern.cp_time2 failed");
+#endif
+	}
 	for (int i = 0; i < CPUSTATES; i++)
-		timeArray[i] = cpu[i];
+#if defined(XOSVIEW_OPENBSD) // aggregates are long, singles uint64_t
+		timeArray[i] = ( cpu ? times[i] : ((long*)(times))[i] );
+#else  // XOSVIEW_FREEBSD || XOSVIEW_NETBSD
+		timeArray[i] = times[cpu * CPUSTATES + i];
 #endif
+#endif
+	free(times);
 }
 
 
@@ -429,7 +476,7 @@ BSDNetInit() {
 }
 
 void
-BSDGetNetInOut(unsigned long long *inbytes, unsigned long long *outbytes, const char *netIface, bool ignored) {
+BSDGetNetInOut(uint64_t *inbytes, uint64_t *outbytes, const char *netIface, bool ignored) {
 	char ifname[IFNAMSIZ];
 	*inbytes = 0;
 	*outbytes = 0;
@@ -482,7 +529,7 @@ BSDGetNetInOut(unsigned long long *inbytes, unsigned long long *outbytes, const
 	while (ifnetp) {
 		bool skipif = false;
 		//  Now, dereference the pointer to get the ifnet struct.
-		safe_kvm_read((u_long)ifnetp, &ifnet, sizeof(ifnet));
+		safe_kvm_read((unsigned long)ifnetp, &ifnet, sizeof(ifnet));
 		strlcpy(ifname, ifnet.if_xname, sizeof(ifname));
 #if defined(XOSVIEW_NETBSD)
 		ifnetp = TAILQ_NEXT(&ifnet, if_list);
@@ -502,7 +549,7 @@ BSDGetNetInOut(unsigned long long *inbytes, unsigned long long *outbytes, const
 			struct ifdata_pcpu ifdata;
 			int ncpus = BSDCountCpus();
 			for (int cpu = 0; cpu < ncpus; cpu++) {
-				safe_kvm_read((u_long)ifdatap + cpu * sizeof(ifdata),
+				safe_kvm_read((unsigned long)ifdatap + cpu * sizeof(ifdata),
 				              &ifdata, sizeof(ifdata));
 				*inbytes  += ifdata.ifd_ibytes;
 				*outbytes += ifdata.ifd_obytes;
@@ -526,7 +573,7 @@ BSDSwapInit() {
 }
 
 void
-BSDGetSwapInfo(u_int64_t *total, u_int64_t *used) {
+BSDGetSwapInfo(uint64_t *total, uint64_t *used) {
 #if defined(HAVE_SWAPCTL)
 	//  This code is based on a patch sent in by Scott Stevens
 	//  (s.k.stevens@ic.ac.uk, at the time).
@@ -549,8 +596,8 @@ BSDGetSwapInfo(u_int64_t *total, u_int64_t *used) {
 	swapiter = sep;
 	bsize = 512;  // block size is that of underlying device, *usually* 512 bytes
 	for ( ; rnswap-- > 0; swapiter++) {
-		*total += (u_int64_t)swapiter->se_nblks * bsize;
-		*used += (u_int64_t)swapiter->se_inuse * bsize;
+		*total += (uint64_t)swapiter->se_nblks * bsize;
+		*used += (uint64_t)swapiter->se_inuse * bsize;
 	}
 	free(sep);
 #else
@@ -560,8 +607,8 @@ BSDGetSwapInfo(u_int64_t *total, u_int64_t *used) {
 	if ( kvm_getswapinfo(kd, &kswap, 1, 0) )
 		err(EX_OSERR, "BSDGetSwapInfo(): kvm_getswapinfo failed");
 
-	*total = (u_int64_t)kswap.ksw_total * pgsize;
-	*used = (u_int64_t)kswap.ksw_used * pgsize;
+	*total = (uint64_t)kswap.ksw_total * pgsize;
+	*used = (uint64_t)kswap.ksw_used * pgsize;
 #endif
 }
 
@@ -670,11 +717,11 @@ DevStat_Init(void) {
 	}
 }
 
-u_int64_t
-DevStat_Get(u_int64_t *read_bytes, u_int64_t *write_bytes) {
+uint64_t
+DevStat_Get(uint64_t *read_bytes, uint64_t *write_bytes) {
 	register int dn;
 	long double busy_seconds;
-	u_int64_t reads, writes, total_bytes = 0;
+	uint64_t reads, writes, total_bytes = 0;
 	struct devinfo *tmp_dinfo;
 
 	if (nodisk > 0)
@@ -798,8 +845,8 @@ BSDDiskInit() {
 	return 1;
 }
 
-u_int64_t
-BSDGetDiskXFerBytes(u_int64_t *read_bytes, u_int64_t *write_bytes) {
+uint64_t
+BSDGetDiskXFerBytes(uint64_t *read_bytes, uint64_t *write_bytes) {
 #if defined(HAVE_DEVSTAT)
 	return DevStat_Get(read_bytes, write_bytes);
 #else
@@ -836,7 +883,7 @@ BSDGetDiskXFerBytes(u_int64_t *read_bytes, u_int64_t *write_bytes) {
 	safe_kvm_read_symbol(DISKLIST_SYM_INDEX, &kvmdisklist, sizeof(kvmdisklist));
 	kvmdiskptr = TAILQ_FIRST(&kvmdisklist);
 	while (kvmdiskptr != NULL) {
-		safe_kvm_read((u_long)kvmdiskptr, &kvmcurrdisk, sizeof(kvmcurrdisk));
+		safe_kvm_read((unsigned long)kvmdiskptr, &kvmcurrdisk, sizeof(kvmcurrdisk));
 		*read_bytes += kvmcurrdisk.dk_rbytes;
 		*write_bytes += kvmcurrdisk.dk_wbytes;
 		kvmdiskptr = TAILQ_NEXT(&kvmcurrdisk, dk_link);
@@ -855,7 +902,7 @@ BSDIntrInit() {
 	// Make sure the intr counter array is nonzero in size.
 #if defined(XOSVIEW_FREEBSD)
 # if __FreeBSD_version >= 900040
-	int nintr;
+	size_t nintr;
 	safe_kvm_read(nlst[EINTRCNT_SYM_INDEX].n_value, &nintr, sizeof(nintr));
 	return ValidSymbol(INTRCNT_SYM_INDEX) && ValidSymbol(EINTRCNT_SYM_INDEX) && (nintr > 0);
 # else
@@ -872,8 +919,7 @@ BSDNumInts() {
 	/* This code is stolen from vmstat. */
 	int count = 0, nbr = 0;
 #if defined(XOSVIEW_FREEBSD)
-	int nintr = 0;
-	size_t inamlen;
+	size_t inamlen, nintr;
 	char *intrnames, *intrs;
 
 # if __FreeBSD_version >= 900040
@@ -888,13 +934,13 @@ BSDNumInts() {
 		return 0;
 	}
 
-	intrnames = intrs = (char *)malloc((size_t)inamlen);
+	intrnames = intrs = (char *)malloc(inamlen);
 	if (!intrs)
 		err(EX_OSERR, "BSDNumInts(): malloc failed");
-	safe_kvm_read(nlst[INTRNAMES_SYM_INDEX].n_value, intrs, (size_t)inamlen);
+	safe_kvm_read(nlst[INTRNAMES_SYM_INDEX].n_value, intrs, inamlen);
 	nintr /= sizeof(long);
-	while (--nintr >= 0) {
-		if ( sscanf(intrnames, "irq%d", &nbr) && nbr > count )
+	for (uint i = 0; i < nintr; i++) {
+		if ( intrnames[0] && sscanf(intrnames, "irq%d", &nbr) == 1 && nbr > count )
 			count = nbr;
 		intrnames += strlen(intrnames) + 1;
 	}
@@ -908,11 +954,11 @@ BSDNumInts() {
 	safe_kvm_read(nlst[ALLEVENTS_SYM_INDEX].n_value, &events, sizeof(events));
 	evptr = TAILQ_FIRST(&events);
 	while (evptr) {
-		safe_kvm_read((u_long)evptr, &evcnt, sizeof(evcnt));
+		safe_kvm_read((unsigned long)evptr, &evcnt, sizeof(evcnt));
 		if (evcnt.ev_type == EVCNT_TYPE_INTR) {
 			if ( !(name = (char *)malloc(evcnt.ev_namelen + 1)) )
 				err(EX_OSERR, "BSDNumInts(): malloc failed");
-			safe_kvm_read((u_long)evcnt.ev_name, name, evcnt.ev_namelen + 1);
+			safe_kvm_read((unsigned long)evcnt.ev_name, name, evcnt.ev_namelen + 1);
 			if ( sscanf(name, "%s%d", dummy, &nbr) == 2 && nbr > count )
 				count = nbr;
 			free(name);
@@ -972,14 +1018,13 @@ BSDNumInts() {
 }
 
 void
-BSDGetIntrStats(unsigned long *intrCount, unsigned int *intrNbrs) {
+BSDGetIntrStats(uint64_t *intrCount, unsigned int *intrNbrs) {
 	/* This code is stolen from vmstat */
 	int nbr = 0;
 #if defined(XOSVIEW_FREEBSD)
 	unsigned long *kvm_intrcnt, *intrcnt;
 	char *kvm_intrnames, *intrnames;
-	size_t inamlen;
-	int nintr = 0;
+	size_t inamlen, nintr;
 
 # if __FreeBSD_version >= 900040
 	safe_kvm_read(nlst[EINTRCNT_SYM_INDEX].n_value, &nintr, sizeof(nintr));
@@ -1007,9 +1052,9 @@ BSDGetIntrStats(unsigned long *intrCount, unsigned int *intrNbrs) {
 	/* kvm_intrname has the ASCII names of the IRQs, every null-terminated
 	 * string corresponds to a value in the kvm_intrcnt array
 	 * e.g. irq1: atkbd0   */
-	while (--nintr >= 0) {
+	for (uint i = 0; i < nintr; i++) {
 		/* Figure out which irq we have here */
-		if ( sscanf(kvm_intrnames, "irq%d", &nbr) == 1 ) {
+		if ( kvm_intrnames[0] && sscanf(kvm_intrnames, "irq%d", &nbr) == 1 ) {
 			intrCount[nbr] = *kvm_intrcnt;
 			if (intrNbrs)
 				intrNbrs[nbr] = 1;
@@ -1028,11 +1073,11 @@ BSDGetIntrStats(unsigned long *intrCount, unsigned int *intrNbrs) {
 	safe_kvm_read(nlst[ALLEVENTS_SYM_INDEX].n_value, &events, sizeof(events));
 	evptr = TAILQ_FIRST(&events);
 	while (evptr) {
-		safe_kvm_read((u_long)evptr, &evcnt, sizeof(evcnt));
+		safe_kvm_read((unsigned long)evptr, &evcnt, sizeof(evcnt));
 		if (evcnt.ev_type == EVCNT_TYPE_INTR) {
 			if ( !(name = (char *)malloc(evcnt.ev_namelen + 1)) )
 				err(EX_OSERR, "BSDGetIntrStats(): malloc failed");
-			safe_kvm_read((u_long)evcnt.ev_name, name, evcnt.ev_namelen + 1);
+			safe_kvm_read((unsigned long)evcnt.ev_name, name, evcnt.ev_namelen + 1);
 			if ( sscanf(name, "%s%d", dummy, &nbr) == 2 ) {
 				intrCount[nbr] = evcnt.ev_count;
 				if (intrNbrs)
@@ -1044,7 +1089,7 @@ BSDGetIntrStats(unsigned long *intrCount, unsigned int *intrNbrs) {
 	}
 #elif defined(XOSVIEW_OPENBSD)
 	int nintr = 0;
-	u_quad_t count = 0;
+	uint64_t count = 0;
 	size_t size = sizeof(nintr);
 	int mib_int[4] = { CTL_KERN, KERN_INTRCNT, KERN_INTRCNT_NUM };
 	if ( sysctl(mib_int, 3, &nintr, &size, NULL, 0) < 0 ) {
@@ -1132,10 +1177,10 @@ BSDCountCpus(void) {
 	return cpus;
 }
 
-void
+#if defined(__i386__) || defined(__x86_64__)
+unsigned int
 BSDGetCPUTemperature(float *temps, float *tjmax) {
-	if (!temps)
-		errx(EX_SOFTWARE, "NULL pointer passed to BSDGetCPUTemperature().");
+	unsigned int nbr = 0;
 #if defined(XOSVIEW_NETBSD)
 	// All kinds of sensors are read with libprop. We have to go through them
 	// to find either Intel Core 2 or AMD ones. Actual temperature is in
@@ -1151,7 +1196,7 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 
 	if ( (fd = open(_PATH_SYSMON, O_RDONLY)) == -1 ) {
 		warn("Could not open %s", _PATH_SYSMON);
-		return;  // this seems to happen occasionally, so only warn
+		return 0;  // this seems to happen occasionally, so only warn
 	}
 	if (prop_dictionary_recv_ioctl(fd, ENVSYS_GETDICTIONARY, &pdict))
 		err(EX_OSERR, "Could not get sensor dictionary");
@@ -1160,7 +1205,7 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 
 	if (prop_dictionary_count(pdict) == 0) {
 		warn("No sensors found");
-		return;
+		return 0;
 	}
 	if ( !(piter = prop_dictionary_iterator(pdict)) )
 		err(EX_OSERR, "Could not get sensor iterator");
@@ -1180,8 +1225,11 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 		while ( (pobj = prop_object_iterator_next(piter2)) ) {
 			if ( !(pobj1 = prop_dictionary_get((prop_dictionary_t)pobj, "type")) )
 				continue;
-			if ( (pobj1 = prop_dictionary_get((prop_dictionary_t)pobj, "cur-value")) )
-				temps[i] = (prop_number_integer_value((prop_number_t)pobj1) / 1000000.0) - 273.15;
+			if ( (pobj1 = prop_dictionary_get((prop_dictionary_t)pobj, "cur-value")) ) {
+				if (temps)
+					temps[i] = (prop_number_integer_value((prop_number_t)pobj1) / 1000000.0) - 273.15;
+				nbr++;
+			}
 			if ( (pobj2 = prop_dictionary_get((prop_dictionary_t)pobj, "critical-max")) && tjmax )
 				tjmax[i] = (prop_number_integer_value((prop_number_t)pobj2) / 1000000.0) - 273.15;
 		}
@@ -1199,7 +1247,7 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 	// Values are in microdegrees Kelvin.
 	struct sensordev sd;
 	struct sensor s;
-	int nbr = 0;
+	int cpu = 0;
 	char dummy[10];
 
 	for (int dev = 0; dev < 1024; dev++) {  // go through all sensor devices
@@ -1214,7 +1262,7 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 		}
 		if ( strncmp(sd.xname, "cpu", 3) )
 			continue;  // not CPU sensor
-		sscanf(sd.xname, "%[^0-9]%d", dummy, &nbr);
+		sscanf(sd.xname, "%[^0-9]%d", dummy, &cpu);
 
 		mib_sen[3] = SENSOR_TEMP;  // for each device, get temperature sensors
 		for (int i = 0; i < sd.maxnumt[SENSOR_TEMP]; i++) {
@@ -1224,7 +1272,9 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 				continue;  // no sensor on this core?
 			if (s.flags & SENSOR_FINVALID)
 				continue;
-			temps[nbr] = (float)(s.value - 273150000) / 1000000.0;
+			if (temps)
+				temps[cpu] = (float)(s.value - 273150000) / 1000000.0;
+			nbr++;
 		}
 	}
 #else  /* XOSVIEW_FREEBSD */
@@ -1236,16 +1286,18 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 	int cpus = BSDCountCpus();
 	for (int i = 0; i < cpus; i++) {
 		snprintf(name, 25, "dev.cpu.%d.temperature", i);
-		if ( sysctlbyname(name, &val, &size, NULL, 0) == 0)
+		if ( sysctlbyname(name, &val, &size, NULL, 0) == 0) {
+			nbr++;
+			if (temps)
 #if __FreeBSD_version >= 702106
-			temps[i] = ((float)val - 2732.0) / 10.0;
+				temps[i] = ((float)val - 2732.0) / 10.0;
 #else
-			temps[i] = (float)val;
+				temps[i] = (float)val;
 #endif
-		else {
-			warn("sysctl %s failed", name);
-			temps[i] = -300.0;
 		}
+		else
+			warn("sysctl %s failed", name);
+
 		if (tjmax) {
 			snprintf(name, 25, "dev.cpu.%d.coretemp.tjmax", i);
 			if ( sysctlbyname(name, &val, &size, NULL, 0) == 0 )
@@ -1254,18 +1306,18 @@ BSDGetCPUTemperature(float *temps, float *tjmax) {
 #else
 				tjmax[i] = (float)val;
 #endif
-			else {
+			else
 				warn("sysctl %s failed", name);
-				tjmax[i] = -300.0;
-			}
 		}
 	}
 #endif
 #endif
+	return nbr;
 }
+#endif
 
 void
-BSDGetSensor(const char *name, const char *valname, float *value) {
+BSDGetSensor(const char *name, const char *valname, float *value, char *unit) {
 	if (!name || !valname || !value)
 		errx(EX_SOFTWARE, "NULL pointer passed to BSDGetSensor().");
 #if defined(XOSVIEW_NETBSD)
@@ -1310,16 +1362,48 @@ BSDGetSensor(const char *name, const char *valname, float *value) {
 			val = prop_number_integer_value((prop_number_t)pobj1);
 		else
 			err(EX_USAGE, "Value %s does not exist", valname);
-		if ( strncmp(type, "Temperature", 4) == 0 )
+		if ( strncmp(type, "Temperature", 4) == 0 ) {
 			*value = (val / 1000000.0) - 273.15;  // temperatures are in microkelvins
-		else if ( strncmp(type, "Fan", 3) == 0 ||
-		          strncmp(type, "Integer", 3) == 0 )
-			*value = (float)val;                  // plain integer values
-		else if ( strncmp(type, "Voltage", 4) == 0 ||
-		          strncmp(type, "Ampere", 4) == 0  ||
-		          strncmp(type, "Watt", 4) == 0    ||
-		          strncmp(type, "Ohms", 4) == 0 )
+			if (unit)
+				strcpy(unit, "\260C");
+		}
+		else if ( strncmp(type, "Fan", 3) == 0 ) {
+			*value = (float)val;                  // plain integer value
+			if (unit)
+				strcpy(unit, "RPM");
+		}
+		else if ( strncmp(type, "Integer", 3) == 0 )
+			*value = (float)val;                  // plain integer value
+		else if ( strncmp(type, "Voltage", 4) == 0 ) {
 			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "V");
+		}
+		else if ( strncmp(type, "Ampere hour", 7) == 0 ) {
+			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "Ah");
+		}
+		else if ( strncmp(type, "Ampere", 7) == 0 ) {
+			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "A");
+		}
+		else if ( strncmp(type, "Watt hour", 5) == 0 ) {
+			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "Wh");
+		}
+		else if ( strncmp(type, "Watts", 5) == 0 ) {
+			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "W");
+		}
+		else if ( strncmp(type, "Ohms", 4) == 0 ) {
+			*value = (float)val / 1000000.0;      // electrical units are in micro{V,A,W,Ohm}
+			if (unit)
+				strcpy(unit, "Ohm");
+		}
 	}
 	prop_object_iterator_release(piter);
 	prop_object_release(pdict);
@@ -1337,6 +1421,8 @@ BSDGetSensor(const char *name, const char *valname, float *value) {
 		if ( sysctlbyname(dummy, &val, &size, NULL, 0) < 0 )
 			err(EX_OSERR, "sysctl %s failed", dummy);
 		*value = ((float)val - 2732.0) / 10.0;
+		if (unit)
+			strcpy(unit, "\260C");
 		return;
 	}
 	// If Dragonfly and tzN specified, return. Otherwise, fall through.
@@ -1382,35 +1468,97 @@ BSDGetSensor(const char *name, const char *valname, float *value) {
 				switch (t) {
 				case SENSOR_TEMP:
 					*value = (float)(s.value - 273150000) / 1000000.0;
+					if (unit)
+						strcpy(unit, "\260C");
 					break;
 				case SENSOR_FANRPM:
-				case SENSOR_OHMS:
-				case SENSOR_INDICATOR:
-				case SENSOR_INTEGER:
 					*value = (float)s.value;
+					if (unit)
+						strcpy(unit, "RPM");
 					break;
 				case SENSOR_VOLTS_DC:
 				case SENSOR_VOLTS_AC:
-				case SENSOR_WATTS:
-				case SENSOR_AMPS:
-				case SENSOR_WATTHOUR:
-				case SENSOR_AMPHOUR:
-				case SENSOR_LUX:
-#if defined(XOSVIEW_OPENBSD)
-				case SENSOR_FREQ:
-				case SENSOR_ANGLE:
-#endif
 					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "V");
+					break;
+				case SENSOR_OHMS:
+					*value = (float)s.value;
+					if (unit)
+						strcpy(unit, "Ohm");
+					break;
+				case SENSOR_WATTS:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "W");
+					break;
+				case SENSOR_AMPS:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "A");
+					break;
+				case SENSOR_WATTHOUR:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "Wh");
+					break;
+				case SENSOR_AMPHOUR:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "Ah");
 					break;
 				case SENSOR_PERCENT:
-#if defined(XOSVIEW_OPENBSD)
-				case SENSOR_HUMIDITY:
-#endif
 					*value = (float)s.value / 1000.0;
+					if (unit)
+						strcpy(unit, "%");
+					break;
+				case SENSOR_LUX:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "lx");
 					break;
 				case SENSOR_TIMEDELTA:
 					*value = (float)s.value / 1000000000.0;
+					if (unit)
+						strcpy(unit, "s");
 					break;
+#if defined(XOSVIEW_OPENBSD)
+				case SENSOR_HUMIDITY:
+					*value = (float)s.value / 1000.0;
+					if (unit)
+						strcpy(unit, "%");
+					break;
+				case SENSOR_FREQ:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "Hz");
+					break;
+				case SENSOR_ANGLE:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "\260");
+					break;
+#if OpenBSD > 201211
+				case SENSOR_DISTANCE:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "m");
+					break;
+				case SENSOR_PRESSURE:
+					*value = (float)s.value / 1000.0;
+					if (unit)
+						strcpy(unit, "Pa");
+					break;
+				case SENSOR_ACCEL:
+					*value = (float)s.value / 1000000.0;
+					if (unit)
+						strcpy(unit, "m\\/s\262"); // m/s²
+					break;
+#endif
+#endif
+				case SENSOR_INDICATOR:
+				case SENSOR_INTEGER:
+				case SENSOR_DRIVE:
 				default:
 					*value = (float)s.value;
 					break;
@@ -1622,7 +1770,7 @@ BSDGetBatteryInfo(int *remaining, unsigned int *state) {
 			err(EX_IOERR, "failed to get APM battery info");
 		if ( close(fd) == -1 )
 			err(EX_OSERR, "Could not close %s", APMDEV);
-		if (0 <= aip.ai_batt_life && aip.ai_batt_life <= 100)
+		if (aip.ai_batt_life <= 100)
 			*remaining = aip.ai_batt_life; // only 0-100 are valid values
 		else
 			*remaining = 0;
