@@ -4,11 +4,13 @@
 //  This file may be distributed under terms of the GPL
 //
 
-#include <iostream>
-#include <fstream>
-#include <stdlib.h>
 #include "fieldmeter.h"
-#include "xosview.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <math.h>
+#include <iostream>
+
 
 FieldMeter::FieldMeter( XOSView *parent, int numfields, const char *title,
                         const char *legend, int docaptions, int dolegends,
@@ -19,6 +21,8 @@ FieldMeter::FieldMeter( XOSView *parent, int numfields, const char *title,
      *  CheckResources() themselves.  */
   numWarnings_ = printedZeroTotalMesg_ = 0;
   print_ = PERCENT;
+  metric_ = false;
+  usedoffset_ = 0;
   used_ = 0;
   lastused_ = -1;
   fields_ = NULL;
@@ -61,11 +65,10 @@ void FieldMeter::SetUsedFormat ( const char * const fmt ) {
     print_ = AUTOSCALE;
   else if (!strncasecmp (fmt, "float", 6))
     print_ = FLOAT;
-  else
-  {
-    fprintf (stderr, "Error:  could not parse format of '%s'\n", fmt);
-    fprintf (stderr, "  I expected one of 'percent', 'bytes', or 'float'\n");
-    fprintf (stderr, "  (Case-insensitive)\n");
+  else {
+    std::cerr << "Error:  could not parse format of '" << fmt << "'.\n"
+              << "  I expected one of 'percent', 'autoscale', or 'float'"
+              << " (Case-insensitive)." << std::endl;
     exit(1);
   }
 }
@@ -82,9 +85,8 @@ void FieldMeter::setUsed (double val, double total)
     {
       if (!printedZeroTotalMesg_) {
         printedZeroTotalMesg_ = 1;
-	fprintf(stderr, "Warning:  %s meter had a zero total "
-		"field!  Would have caused a div-by-zero "
-		"exception.\n", name());
+        std::cerr << "Warning: " << name() << " meter had a zero total field! "
+                  << "Would have caused a div-by-zero exception." << std::endl;
       }
       used_ = 0.0;
     }
@@ -92,8 +94,8 @@ void FieldMeter::setUsed (double val, double total)
   else if (print_ == AUTOSCALE)
     used_ = val;
   else {
-    fprintf (stderr, "Error in %s:  I can't handle a "
-		     "UsedType enum value of %d!\n", name(), print_);
+    std::cerr << "Error in " << name() << ":  I can't handle a UsedType enum "
+              << "value of " << print_ << "!" << std::endl;
     exit(1);
   }
 }
@@ -120,9 +122,9 @@ void FieldMeter::draw( void ){
 
     int offset;
     if ( dousedlegends_ )
-      offset = parent_->textWidth( "XXXXXXXXX" );
+      offset = parent_->textWidth( "XXXXXXXXXX" );
     else
-      offset = parent_->textWidth( "XXXXX" );
+      offset = parent_->textWidth( "XXXXXX" );
 
     parent_->drawString( x_ - offset + 1, y_ + height_, title_ );
   }
@@ -145,6 +147,8 @@ void FieldMeter::drawlegend( void ){
   for ( int i = 0 ; i < numfields_ ; i++ ){
     n = 0;
     while ( (*tmp2 != '/') && (*tmp2 != '\0') ){
+      if ( (*tmp2 == '\\') && (*(tmp2 + 1) == '/') ) // allow '/' in field as '\/'
+        memmove( tmp2, tmp2 + 1, strlen(tmp2) );
       tmp2++;
       n++;
     }
@@ -164,83 +168,57 @@ void FieldMeter::drawlegend( void ){
   parent_->setStippleN(0);	/*  Restore default all-bits stipple.  */
 }
 
-void FieldMeter::drawused( int manditory ){
-  if ( !manditory )
+void FieldMeter::drawused( int mandatory ){
+  if ( !mandatory )
     if ( lastused_ == used_ )
       return;
 
   parent_->setStippleN(0);	/*  Use all-bits stipple.  */
   static const int onechar = parent_->textWidth( "X" );
-  static int xoffset = parent_->textWidth( "XXXXX" );
-
+  if (!usedoffset_)  // metric meters need extra space for '-' sign
+    usedoffset_ = ( metric_ ? parent_->textWidth( "XXXXXX" )
+                            : parent_->textWidth( "XXXXX" ) );
   char buf[10];
 
   if (print_ == PERCENT){
     snprintf( buf, 10, "%d%%", (int)used_ );
   }
   else if (print_ == AUTOSCALE){
-    char scale;
-    double scaled_used;
-      /*  Unfortunately, we have to do our comparisons by 1000s (otherwise
-       *  a value of 1020, which is smaller than 1K, could end up
-       *  being printed as 1020, which is wider than what can fit)  */
-      /*  However, we do divide by 1024, so a K really is a K, and not
-       *  1000.  */
-      /*  In addition, we need to compare against 999.5*1000, because
-       *  999.5, if not rounded up to 1.0 K, will be rounded by the
-       *  %.0f to be 1000, which is too wide.  So anything at or above
-       *  999.5 needs to be bumped up.  */
-    if (used_ >= 999.5*1000*1000*1000*1000*1000*1000)
-	{scale='E'; scaled_used = used_/1024/1024/1024/1024/1024/1024;}
-    else if (used_ >= 999.5*1000*1000*1000*1000)
-	{scale='P'; scaled_used = used_/1024/1024/1024/1024/1024;}
-    else if (used_ >= 999.5*1000*1000*1000)
-	{scale='T'; scaled_used = used_/1024/1024/1024/1024;}
-    else if (used_ >= 999.5*1000*1000)
-	{scale='G'; scaled_used = used_/1024/1024/1024;}
-    else if (used_ >= 999.5*1000)
-	{scale='M'; scaled_used = used_/1024/1024;}
-    else if (used_ >= 999.5)
-	{scale='K'; scaled_used = used_/1024;}
-    else {scale='\0'; scaled_used = used_;}
-      /*  For now, we can only print 3 characters, plus the optional
-       *  suffix, without overprinting the legends.  Thus, we can
-       *  print 965, or we can print 34, but we can't print 34.7 (the
-       *  decimal point takes up one character).  bgrayson   */
-      /*  Also check for negative values, and just print "-" for
-       *  them.  */
-    if (scaled_used < 0)
-      snprintf (buf, 10, "-");
-    else if (scaled_used == 0.0)
+    char scale[2];
+    double scaled_used = scaleValue(used_, scale, metric_);
+    /*  For now, we can only print 3 characters, plus the optional sign and
+     *  suffix, without overprinting the legends.  Thus, we can
+     *  print 965, or we can print 34, but we can't print 34.7 (the
+     *  decimal point takes up one character).  bgrayson   */
+    if (scaled_used == 0.0)
       snprintf (buf, 10, "0");
-    else if (scaled_used < 9.95) {
-      //  9.95 or above would get
-      //  rounded to 10.0, which is too wide.
-      if (scale)
-	snprintf (buf, 10, "%.1f%c", scaled_used, scale);
+    else {
+      if (scaled_used < 0 && !metric_)
+        snprintf (buf, 10, "-");
+      else if ( fabs(scaled_used) < 9.95 )
+        //  9.95 or above would get
+        //  rounded to 10.0, which is too wide.
+        snprintf (buf, 10, "%.1f%s", scaled_used, scale);
       else
-	snprintf (buf, 10, "%.1f", scaled_used);
-    } else {
-      if (scale)
-        snprintf (buf, 10, "%.0f%c", scaled_used, scale);
-      else
-        snprintf (buf, 10, "%.0f", scaled_used);
+        snprintf (buf, 10, "%.0f%s", scaled_used, scale);
     }
   }
   else {
-    snprintf( buf, 10, "%.1f", used_ );
+    if ( fabs(used_) < 99.95 )
+      snprintf( buf, 10, "%.1f", used_ );
+    else  // drop the decimal if the string gets too long
+      snprintf( buf, 10, "%.0f", used_ );
   }
 
-  parent_->clear( x_ - xoffset, y_ + height_ - parent_->textHeight(),
-		 xoffset - onechar / 2, parent_->textHeight() + 1 );
+  parent_->clear( x_ - usedoffset_, y_ + height_ - parent_->textHeight(),
+                  usedoffset_ - onechar / 2, parent_->textHeight() + 1 );
   parent_->setForeground( usedcolor_ );
   parent_->drawString( x_ - (strlen( buf ) + 1 ) * onechar + 2,
-		      y_ + height_, buf );
-
+                       y_ + height_, buf );
   lastused_ = used_;
 }
 
-void FieldMeter::drawfields( int manditory ){
+void FieldMeter::drawfields( int mandatory ){
   int twidth, x = x_;
 
   if ( total_ == 0 )
@@ -248,24 +226,23 @@ void FieldMeter::drawfields( int manditory ){
 
   for ( int i = 0 ; i < numfields_ ; i++ ){
     /*  Look for bogus values.  */
-    if (fields_[i] < 0.0) {
+    if (fields_[i] < 0.0 && !metric_) {
       /*  Only print a warning 5 times per meter, followed by a
        *  message about no more warnings.  */
       numWarnings_ ++;
       if (numWarnings_ < 5)
-	fprintf(stderr, "Warning:  meter %s had a negative "
-	  "value of %f for field %d\n", name(), fields_[i], i);
+        std::cerr << "Warning: meter " << name() << " had a negative value of "
+                  << fields_[i] << " for field " << i << std::endl;
       if (numWarnings_ == 5)
-        fprintf(stderr, "Future warnings from the %s meter "
-	  "will not be displayed.\n", name());
+        std::cerr << "Future warnings from the " << name() << " meter will not "
+                  << "be displayed." << std::endl;
     }
 
-    twidth = (int) ((width_ * (double) fields_[i]) / total_);
-//    twidth = (int)((fields_[i] * width_) / total_);
+    twidth = (int)fabs(((width_ * fields_[i]) / total_));
     if ( (i == numfields_ - 1) && ((x + twidth) != (x_ + width_)) )
       twidth = width_ + x_ - x;
 
-    if ( manditory || (twidth != lastvals_[i]) || (x != lastx_[i]) ){
+    if ( mandatory || (twidth != lastvals_[i]) || (x != lastx_[i]) ){
       parent_->setForeground( colors_[i] );
       parent_->setStippleN(i%4);
       parent_->drawFilledRectangle( x, y_, twidth, height_ );
@@ -276,7 +253,7 @@ void FieldMeter::drawfields( int manditory ){
     x += twidth;
   }
   if ( dousedlegends_ )
-    drawused( manditory );
+    drawused( mandatory );
 }
 
 void FieldMeter::checkevent( void ){
