@@ -10,11 +10,15 @@
 //
 
 #include "acpitemp.h"
-#include "xosview.h"
-#include <fstream>
 #include <stdlib.h>
-#include <dirent.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
+#include <fstream>
+#include <iostream>
+#include <string>
 
 static const char PROC_ACPI_TZ[] = "/proc/acpi/thermal_zone";
 static const char SYS_ACPI_TZ[]  = "/sys/devices/virtual/thermal";
@@ -22,6 +26,7 @@ static const char SYS_ACPI_TZ[]  = "/sys/devices/virtual/thermal";
 
 ACPITemp::ACPITemp( XOSView *parent, const char *tempfile, const char *highfile, const char *label, const char *caption )
 : FieldMeter( parent, 3, label, caption, 1, 1, 0 ) {
+  metric_ = true;
   if (!checkacpi(tempfile, highfile)) {
     std::cerr << "Can not find file : ";
     if (tempfile[0] == '/' && highfile[0] == '/')
@@ -143,11 +148,15 @@ void ACPITemp::getacpitemp( void ) {
     if (high > total_)
       total_ = 10 * (int)((high * 1.25) / 10);
     _high = high;
-    snprintf(l, 16, "ACT/%d/%d", (int)high, (int)total_);
+    snprintf(l, 16, "ACT(\260C)/%d/%d", (int)high, (int)total_);
     legend(l);
     do_legend = true;
   }
 
+  setUsed( fields_[0], total_ );
+  if (fields_[0] < 0)
+    fields_[0] = 0.0;
+
   fields_[1] = high - fields_[0];
   if (fields_[1] < 0) { // alarm: T > high
     fields_[1] = 0;
@@ -167,8 +176,6 @@ void ACPITemp::getacpitemp( void ) {
   if (fields_[2] < 0)
     fields_[2] = 0;
 
-  setUsed( fields_[0], total_ );
-
   if (do_legend)
     drawlegend();
 }
